"""Projects command group for YouTrack CLI.

This module provides project management commands for listing, viewing,
and managing YouTrack projects including their metadata and configuration.
"""

import asyncio
from typing import Optional

import click

from ..auth import AuthManager
from ..console import get_console


@click.group()
def projects() -> None:
    """Manage projects - list, view, and configure project settings.

    The projects command group provides access to project-level operations
    including listing available projects, viewing project details, and
    managing project configuration.

    Example:
        yt projects list
        yt projects show PROJECT-ID
    """
    pass


@projects.command(name="list")
@click.option(
    "--fields",
    "-f",
    help="Comma-separated list of project fields to return",
)
@click.option(
    "--top",
    "-t",
    type=int,
    help="Maximum number of projects to return",
)
@click.option(
    "--show-archived",
    is_flag=True,
    help="Include archived projects",
)
@click.option(
    "--format",
    type=click.Choice(["table", "json"]),
    default="table",
    help="Output format",
)
@click.pass_context
def projects_list(
    ctx: click.Context,
    fields: Optional[str],
    top: Optional[int],
    show_archived: bool,
    format: str,
) -> None:
    """List all projects."""
    from ..projects import ProjectManager

    console = get_console()
    auth_manager = AuthManager(ctx.obj.get("config"))
    project_manager = ProjectManager(auth_manager)

    console.print("📋 Fetching projects...", style="blue")

    try:
        result = asyncio.run(project_manager.list_projects(fields=fields, top=top, show_archived=show_archived))

        if result["status"] == "success":
            projects = result["data"]

            if format == "table":
                project_manager.display_projects_table(projects)
                console.print(f"\n[dim]Total: {result['count']} projects[/dim]")
            else:
                import json

                console.print(json.dumps(projects, indent=2))
        else:
            console.print(f"❌ {result['message']}", style="red")
            raise click.ClickException("Failed to list projects")

    except Exception as e:
        console.print(f"❌ Error listing projects: {e}", style="red")
        raise click.ClickException("Failed to list projects") from e


@projects.command(name="create")
@click.argument("name")
@click.argument("short_name")
@click.option(
    "--leader",
    "-l",
    help="Project leader username (e.g., 'admin', 'ryan') or ID (e.g., '2-3')",
)
@click.option(
    "--description",
    "-d",
    help="Project description",
)
@click.option(
    "--template",
    "-t",
    type=click.Choice(["scrum", "kanban"]),
    help="Project template",
)
@click.pass_context
def projects_create(
    ctx: click.Context,
    name: str,
    short_name: str,
    leader: Optional[str],
    description: Optional[str],
    template: Optional[str],
) -> None:
    """Create a new project.

    Creates a new project with the specified name and short name.
    Both name and short_name are required positional arguments.

    Examples:
        # Create a basic project (will prompt for leader)
        yt projects create "CLI Testing Project" CLI-TEST

        # Create a project with all options
        yt projects create "My Project" MP --leader admin --description "Project description" --template scrum

    Note: Both NAME and SHORT_NAME are required positional arguments.
    The leader will be prompted interactively if not specified with --leader.
    """
    from ..projects import ProjectManager

    console = get_console()
    auth_manager = AuthManager(ctx.obj.get("config"))
    project_manager = ProjectManager(auth_manager)

    # Prompt for leader if not provided (maintains backward compatibility)
    if not leader:
        leader = click.prompt("Project leader username (e.g., 'admin', 'ryan') or ID (e.g., '2-3')")

    console.print(f"🚀 Creating project '{name}'...", style="blue")

    try:
        result = asyncio.run(
            project_manager.create_project(
                name=name,
                short_name=short_name,
                leader_id=leader,
                description=description,
                template=template,
            )
        )

        if result["status"] == "success":
            console.print(f"✅ {result['message']}", style="green")
            project = result["data"]
            console.print(f"Project ID: {project.get('id', 'N/A')}", style="blue")
            console.print(f"Short Name: {project.get('shortName', 'N/A')}", style="blue")
        else:
            console.print(f"❌ {result['message']}", style="red")
            raise click.ClickException("Failed to create project")

    except Exception as e:
        console.print(f"❌ Error creating project: {e}", style="red")
        raise click.ClickException("Failed to create project") from e


@projects.command()
@click.argument("project_id")
@click.option(
    "--name",
    "-n",
    help="New project name",
)
@click.option(
    "--description",
    "-d",
    help="New project description",
)
@click.option(
    "--leader",
    "-l",
    help="New project leader username (e.g., 'admin', 'ryan') or ID (e.g., '2-3')",
)
@click.option(
    "--show-details",
    is_flag=True,
    help="Show detailed project information",
)
@click.pass_context
def configure(
    ctx: click.Context,
    project_id: str,
    name: Optional[str],
    description: Optional[str],
    leader: Optional[str],
    show_details: bool,
) -> None:
    """Configure project settings."""
    from ..projects import ProjectManager

    console = get_console()
    auth_manager = AuthManager(ctx.obj.get("config"))
    project_manager = ProjectManager(auth_manager)

    if show_details:
        console.print(f"📋 Fetching project '{project_id}' details...", style="blue")

        try:
            result = asyncio.run(project_manager.get_project(project_id))

            if result["status"] == "success":
                project_manager.display_project_details(result["data"])
            else:
                console.print(f"❌ {result['message']}", style="red")
                raise click.ClickException("Failed to get project details")

        except Exception as e:
            console.print(f"❌ Error getting project details: {e}", style="red")
            raise click.ClickException("Failed to get project details") from e
    else:
        if not any([name, description, leader]):
            console.print("❌ No configuration changes specified.", style="red")
            console.print(
                "Use --name, --description, or --leader options, or --show-details to view current settings.",
                style="blue",
            )
            return

        console.print(f"⚙️  Updating project '{project_id}'...", style="blue")

        try:
            result = asyncio.run(
                project_manager.update_project(
                    project_id=project_id,
                    name=name,
                    description=description,
                    leader_id=leader,
                )
            )

            if result["status"] == "success":
                console.print(f"✅ {result['message']}", style="green")
                project = result["data"]
                console.print(f"Name: {project.get('name', 'N/A')}", style="blue")
                leader_name = project.get("leader", {}).get("fullName", "N/A")
                console.print(f"Leader: {leader_name}", style="blue")
            else:
                console.print(f"❌ {result['message']}", style="red")
                raise click.ClickException("Failed to update project")

        except Exception as e:
            console.print(f"❌ Error updating project: {e}", style="red")
            raise click.ClickException("Failed to update project") from e


@projects.command()
@click.argument("project_id")
@click.option(
    "--force",
    is_flag=True,
    help="Skip confirmation prompt",
)
@click.pass_context
def archive(
    ctx: click.Context,
    project_id: str,
    force: bool,
) -> None:
    """Archive a project."""
    from ..projects import ProjectManager

    console = get_console()
    auth_manager = AuthManager(ctx.obj.get("config"))
    project_manager = ProjectManager(auth_manager)

    if not force:
        confirmation_msg = f"Are you sure you want to archive project '{project_id}'?"
        if not click.confirm(confirmation_msg):
            console.print("Archive cancelled.", style="yellow")
            return

    console.print(f"📦 Archiving project '{project_id}'...", style="blue")

    try:
        result = asyncio.run(project_manager.archive_project(project_id))

        if result["status"] == "success":
            console.print(f"✅ {result['message']}", style="green")
            console.print("Project has been archived and is no longer active.", style="yellow")
        else:
            console.print(f"❌ {result['message']}", style="red")
            raise click.ClickException("Failed to archive project")

    except Exception as e:
        console.print(f"❌ Error archiving project: {e}", style="red")
        raise click.ClickException("Failed to archive project") from e


@click.command(name="list")
@click.argument("project_id")
@click.option(
    "--fields",
    "-f",
    help="Comma-separated list of custom field attributes to return",
)
@click.option(
    "--top",
    "-t",
    type=int,
    help="Maximum number of custom fields to return",
)
@click.option(
    "--format",
    type=click.Choice(["table", "json"]),
    default="table",
    help="Output format",
)
@click.pass_context
def fields_list(
    ctx: click.Context,
    project_id: str,
    fields: Optional[str],
    top: Optional[int],
    format: str,
) -> None:
    """List custom fields for a project."""
    from ..projects import ProjectManager

    console = get_console()
    auth_manager = AuthManager(ctx.obj.get("config"))
    project_manager = ProjectManager(auth_manager)

    console.print(f"📋 Fetching custom fields for project '{project_id}'...", style="blue")

    try:
        result = asyncio.run(project_manager.list_custom_fields(project_id=project_id, fields=fields, top=top))

        if result["status"] == "success":
            custom_fields = result["data"]

            if format == "table":
                project_manager.display_custom_fields_table(custom_fields)
                console.print(f"\n[dim]Total: {result['count']} custom fields[/dim]")
            else:
                import json

                console.print(json.dumps(custom_fields, indent=2))
        else:
            console.print(f"❌ {result['message']}", style="red")
            raise click.ClickException("Failed to list custom fields")

    except Exception as e:
        console.print(f"❌ Error listing custom fields: {e}", style="red")
        raise click.ClickException("Failed to list custom fields") from e


@click.command(name="attach")
@click.argument("project_id")
@click.argument("field_id")
@click.option(
    "--type",
    "field_type",
    required=True,
    type=click.Choice(
        [
            "EnumProjectCustomField",
            "MultiEnumProjectCustomField",
            "SingleUserProjectCustomField",
            "MultiUserProjectCustomField",
            "SimpleProjectCustomField",
            "VersionProjectCustomField",
            "MultiVersionProjectCustomField",
            "DateProjectCustomField",
            "IntegerProjectCustomField",
            "FloatProjectCustomField",
            "BooleanProjectCustomField",
        ]
    ),
    help="Type of project custom field to create",
)
@click.option(
    "--required",
    is_flag=True,
    help="Make the field required (cannot be empty)",
)
@click.option(
    "--empty-text",
    help="Text to display when field is empty",
)
@click.option(
    "--private",
    is_flag=True,
    help="Make the field private (not visible to all users)",
)
@click.pass_context
def fields_attach(
    ctx: click.Context,
    project_id: str,
    field_id: str,
    field_type: str,
    required: bool,
    empty_text: Optional[str],
    private: bool,
) -> None:
    """Attach an existing custom field to a project.

    PROJECT_ID: The project ID or short name
    FIELD_ID: The global custom field ID to attach
    """
    from ..projects import ProjectManager

    console = get_console()
    auth_manager = AuthManager(ctx.obj.get("config"))
    project_manager = ProjectManager(auth_manager)

    console.print(f"🔗 Attaching custom field '{field_id}' to project '{project_id}'...", style="blue")

    try:
        result = asyncio.run(
            project_manager.attach_custom_field(
                project_id=project_id,
                field_id=field_id,
                field_type=field_type,
                can_be_empty=not required,
                empty_field_text=empty_text,
                is_public=not private,
            )
        )

        if result["status"] == "success":
            console.print(f"✅ {result['message']}", style="green")
            field_data = result["data"]
            field_name = field_data.get("field", {}).get("name", "N/A")
            console.print(f"Field Name: {field_name}", style="blue")
            console.print(f"Required: {'Yes' if not field_data.get('canBeEmpty', True) else 'No'}", style="blue")
        else:
            console.print(f"❌ {result['message']}", style="red")
            raise click.ClickException("Failed to attach custom field")

    except Exception as e:
        console.print(f"❌ Error attaching custom field: {e}", style="red")
        raise click.ClickException("Failed to attach custom field") from e


@click.command(name="update")
@click.argument("project_id")
@click.argument("field_id")
@click.option(
    "--required/--optional",
    default=None,
    help="Make the field required or optional",
)
@click.option(
    "--empty-text",
    help="Text to display when field is empty",
)
@click.option(
    "--public/--private",
    default=None,
    help="Make the field public or private",
)
@click.pass_context
def fields_update(
    ctx: click.Context,
    project_id: str,
    field_id: str,
    required: Optional[bool],
    empty_text: Optional[str],
    public: Optional[bool],
) -> None:
    """Update settings of a custom field in a project.

    PROJECT_ID: The project ID or short name
    FIELD_ID: The project custom field ID to update
    """
    from ..projects import ProjectManager

    console = get_console()
    auth_manager = AuthManager(ctx.obj.get("config"))
    project_manager = ProjectManager(auth_manager)

    if required is None and empty_text is None and public is None:
        console.print("❌ No updates specified.", style="red")
        console.print(
            "Use --required/--optional, --empty-text, or --public/--private options.",
            style="blue",
        )
        return

    console.print(f"⚙️  Updating custom field '{field_id}' in project '{project_id}'...", style="blue")

    try:
        result = asyncio.run(
            project_manager.update_custom_field(
                project_id=project_id,
                field_id=field_id,
                can_be_empty=not required if required is not None else None,
                empty_field_text=empty_text,
                is_public=public,
            )
        )

        if result["status"] == "success":
            console.print(f"✅ {result['message']}", style="green")
            field_data = result["data"]
            field_name = field_data.get("field", {}).get("name", "N/A")
            console.print(f"Field Name: {field_name}", style="blue")
            console.print(f"Required: {'Yes' if not field_data.get('canBeEmpty', True) else 'No'}", style="blue")
            console.print(f"Visibility: {'Public' if field_data.get('isPublic', True) else 'Private'}", style="blue")
        else:
            console.print(f"❌ {result['message']}", style="red")
            raise click.ClickException("Failed to update custom field")

    except Exception as e:
        console.print(f"❌ Error updating custom field: {e}", style="red")
        raise click.ClickException("Failed to update custom field") from e


@click.command(name="detach")
@click.argument("project_id")
@click.argument("field_id")
@click.option(
    "--force",
    is_flag=True,
    help="Skip confirmation prompt",
)
@click.pass_context
def fields_detach(
    ctx: click.Context,
    project_id: str,
    field_id: str,
    force: bool,
) -> None:
    """Remove a custom field from a project.

    PROJECT_ID: The project ID or short name
    FIELD_ID: The project custom field ID to remove
    """
    from ..projects import ProjectManager

    console = get_console()
    auth_manager = AuthManager(ctx.obj.get("config"))
    project_manager = ProjectManager(auth_manager)

    if not force:
        confirmation_msg = f"Are you sure you want to remove custom field '{field_id}' from project '{project_id}'?"
        if not click.confirm(confirmation_msg):
            console.print("Operation cancelled.", style="yellow")
            return

    console.print(f"🗑️  Removing custom field '{field_id}' from project '{project_id}'...", style="blue")

    try:
        result = asyncio.run(project_manager.detach_custom_field(project_id, field_id))

        if result["status"] == "success":
            console.print(f"✅ {result['message']}", style="green")
        else:
            console.print(f"❌ {result['message']}", style="red")
            raise click.ClickException("Failed to remove custom field")

    except Exception as e:
        console.print(f"❌ Error removing custom field: {e}", style="red")
        raise click.ClickException("Failed to remove custom field") from e


# Create a simple fields group
@click.group()
def fields() -> None:
    """Manage project custom fields.

    This command group provides access to project custom field operations
    including listing, attaching, updating, and removing custom fields.

    Example:
        yt projects fields list PROJECT-ID
        yt projects fields attach PROJECT-ID FIELD-ID
    """
    pass


# Add all the commands to the group
fields.add_command(fields_list)
fields.add_command(fields_attach)
fields.add_command(fields_update)
fields.add_command(fields_detach)

# Add the fields command group to the projects group
projects.add_command(fields)
